@@ -1,8 +1,10 @@
-TVAR.LRtest <- function (data, lag=1, trend=TRUE, series, thDelay = 1:m, mTh=1, thVar, nboot=10, plot=FALSE, trim=0.1, test=c("1vs", "2vs3"), check=FALSE, model=c("TAR", "MTAR")) {
+TVAR.LRtest <- function (data, lag=1, trend=TRUE, series, thDelay = 1:m, mTh=1, thVar, nboot=10, plot=FALSE, trim=0.1, test=c("1vs", "2vs3"), model=c("TAR", "MTAR"), hpc=c("none", "foreach"), trace=FALSE, check=FALSE) {
 
 ##Check args
 test<-match.arg(test)
 model<-match.arg(model)
+hpc<-match.arg(hpc)
+
 if (missing(series))  series <- deparse(substitute(data))
 
 if(is.null(colnames(data)))
@@ -174,7 +176,7 @@ posBest<-which(result==min(result, na.rm=TRUE))
 bestDelay<-IDS[posBest,1]
 bestThresh<-IDS[posBest,2]
 
-cat("Best unique threshold", bestThresh, "\t\t\t\t SSR", min(result, na.rm=TRUE), "\n")
+if(trace) cat("Best unique threshold", bestThresh, "\t\t\t\t SSR", min(result, na.rm=TRUE), "\n")
 
 Sigma_mod1thresh<-Sigma_1thresh(gam1=bestThresh, d=bestDelay,Z=Z,Y=Y, trans=z)
 ##################
@@ -226,8 +228,10 @@ Thresh3<-condiStep(allgammas, Thresh2, fun=SSR_2thresh, MoreArgs=More)
 smallThresh<-min(Thresh2, Thresh3$newThresh)
 bigThresh<-max(Thresh2, Thresh3$newThresh)
 
-cat("Second best: ",Thresh2, " (conditionnal on ",bestThresh, ")\n")
-cat("Iterative best: ",Thresh3$newThresh, " (conditionnal on ",Thresh2, ")\n")
+if(trace){
+  cat("Second best: ",Thresh2, " (conditionnal on ",bestThresh, ")\n")
+  cat("Iterative best: ",Thresh3$newThresh, " (conditionnal on ",Thresh2, ")\n")
+}
 
 Sigma_mod2thresh<-Sigma_2thresh(gam1=smallThresh,gam2=bigThresh,d=bestDelay, Z=Z, Y=Y,trans=z)
 
@@ -386,7 +390,12 @@ bootstraploop<-function(y, thVar=NULL){
 
 
 
-LRtestboot<-replicate(n=nboot,bootstraploop(y=resids))
+LRtestboot<-if(hpc=="none"){
+  replicate(n=nboot,bootstraploop(y=resids))
+} else {
+  foreach(i=1:nboot, .export="bootstraploop", .combine="rbind") %dopar% bootstraploop(y=resids)
+}
+
 LRtestboot12<-unlist(LRtestboot[1,])
 LRtestboot13<-unlist(LRtestboot[2,])
 LRtestboot23<-unlist(LRtestboot[3,])
