@@ -153,7 +153,8 @@ vec2var.tsDyn <- function(x){
 
 ## vecm
   ecdet <- if(model=="VECM") x$model.specific$LRinclude else "none"
-  vecm<- new("ca.jo", season = NULL, dumvar=NULL, ecdet=ecdet,lag=as.integer(lag),spec="transitory")
+  aChar <- "fakeChar"
+  vecm<- new("ca.jo", season = NULL, dumvar=NULL, ecdet=ecdet,lag=as.integer(lag),spec="transitory", lambda=aChar)
 
 ## datamat
   if(model=="VAR"){
